@@ -1,5 +1,5 @@
 #
-# Copyright 2018-2020 IBM Corporation
+# Copyright 2018-2021 Elyra Authors
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,11 +14,17 @@
 # limitations under the License.
 #
 import os
+
+import nbformat
 import pytest
 
-from elyra.pipeline import PipelineParser
+from elyra.pipeline.parser import PipelineParser
+from elyra.pipeline.pipeline import GenericOperation
 from elyra.pipeline.processor_local import LocalPipelineProcessor
-from .util import _read_pipeline_resource, NotebookNode, PythonNode, construct_pipeline
+from elyra.pipeline.tests.util import _read_pipeline_resource
+from elyra.pipeline.tests.util import construct_pipeline
+from elyra.pipeline.tests.util import NotebookNode
+from elyra.pipeline.tests.util import PythonNode
 
 
 @pytest.fixture
@@ -30,9 +36,9 @@ def pipeline_dir(tmp_path):
 
 def test_pipeline_execution_order_in_complex_pipeline():
     expected_operation_names = ['a', 'b', 'c', 'd', 'e', 'f', 'x', 'y', 'g', 'h']
-    pipeline_definitions = _read_pipeline_resource('resources/sample_pipelines/pipeline_dependency_complex.json')
+    pipeline_json = _read_pipeline_resource('resources/sample_pipelines/pipeline_dependency_complex.json')
 
-    pipeline = PipelineParser().parse(pipeline_definitions)
+    pipeline = PipelineParser().parse(pipeline_json)
     current_ordered_operation_names = _get_operation_names(pipeline.operations.values())
     assert current_ordered_operation_names != expected_operation_names
 
@@ -46,9 +52,9 @@ def test_pipeline_execution_order_in_complex_pipeline():
 
 def test_pipeline_execution_order_in_simple_pipeline():
     expected_operation_names = ['f', 'a', 'c', 'g']
-    pipeline_definitions = _read_pipeline_resource('resources/sample_pipelines/pipeline_dependency_simple.json')
+    pipeline_json = _read_pipeline_resource('resources/sample_pipelines/pipeline_dependency_simple.json')
 
-    pipeline = PipelineParser().parse(pipeline_definitions)
+    pipeline = PipelineParser().parse(pipeline_json)
     current_ordered_operation_names = _get_operation_names(pipeline.operations.values())
     assert current_ordered_operation_names != expected_operation_names
 
@@ -64,11 +70,12 @@ def test_pipeline_get_envs():
 
     # Ensure pipeline operation env lists are properly converted to dictionaries.
 
-    pipeline_definitions = _read_pipeline_resource('resources/sample_pipelines/pipeline_dependency_complex.json')
+    pipeline_json = _read_pipeline_resource('resources/sample_pipelines/pipeline_dependency_complex.json')
 
-    pipeline = PipelineParser().parse(pipeline_definitions)
+    pipeline = PipelineParser().parse(pipeline_json)
 
     for op in pipeline.operations.values():
+        assert isinstance(op, GenericOperation)
         op_envs = op.env_vars_as_dict()
         assert op_envs['OP_NAME'] == op.name
 
@@ -93,6 +100,29 @@ def test_pipeline_execution(pipeline_dir):
             assert os.path.exists(os.path.join(pipeline_dir, output))
 
 
+def test_pipeline_execution_missing_kernelspec(pipeline_dir):
+    # Construct 4-node pipeline consisting of 3 notebooks and 1 python script.
+    # This pipeline is "diamond shaped" with node1 feeding nodes 2 and 3, each then
+    # feeding node4.
+    node1 = NotebookNode("node1", num_outputs=2)
+    node2 = PythonNode("node2", num_outputs=2, input_nodes=[node1])
+    node3 = NotebookNode("node3", num_outputs=2, input_nodes=[node1])
+    node4 = NotebookNode("node4", num_outputs=2, input_nodes=[node2, node3])
+    nodes = [node1, node2, node3, node4]
+
+    pipeline = construct_pipeline("p1", nodes=nodes, location=pipeline_dir)
+
+    node1nb_file = os.path.join(pipeline_dir, pipeline.operations[node1.id].filename)
+    nb = nbformat.read(node1nb_file, 4)
+    nb['metadata'].pop('kernelspec')
+    nbformat.write(nb, node1nb_file)
+
+    with pytest.raises(RuntimeError) as e:
+        LocalPipelineProcessor(pipeline_dir).process(pipeline)
+    assert 'Error processing operation node1 (node1.ipynb): No kernel ' \
+           'name found in notebook and no override provided.' in str(e.value)
+
+
 def test_pipeline_execution_bad_notebook(pipeline_dir):
     # Construct 4-node pipeline where node 3 (nodebook) produces a failure
     node1 = NotebookNode("node1", num_outputs=2)
@@ -107,7 +137,7 @@ def test_pipeline_execution_bad_notebook(pipeline_dir):
 
     with pytest.raises(RuntimeError) as e:
         LocalPipelineProcessor(pipeline_dir).process(pipeline)
-    assert str(e.value) == 'Error processing operation node3.'
+    assert 'Error processing operation node3' in str(e.value)
 
     # Confirm outputs (and non-outputs)
     for node in processed_nodes:
@@ -133,7 +163,7 @@ def test_pipeline_execution_bad_python(pipeline_dir):
 
     with pytest.raises(RuntimeError) as e:
         LocalPipelineProcessor(pipeline_dir).process(pipeline)
-    assert str(e.value) == 'Error processing operation node2.'
+    assert 'Error processing operation node2' in str(e.value)
 
     # Confirm outputs (and non-outputs)
     for node in processed_nodes:
