@@ -33,7 +33,7 @@ from elyra.pipeline.tests.test_pipeline_parser import _read_pipeline_resource
 
 
 @pytest.fixture
-def processor():
+def processor(setup_factory_data):
     processor = KfpPipelineProcessor(os.getcwd())
     return processor
 
@@ -195,7 +195,10 @@ def test_collect_envs(processor):
 
 def test_process_list_value_function(processor):
     # Test values that will be successfully converted to list
+    assert processor._process_list_value("") == []
+    assert processor._process_list_value(None) == []
     assert processor._process_list_value("[]") == []
+    assert processor._process_list_value("None") == []
     assert processor._process_list_value("['elem1']") == ["elem1"]
     assert processor._process_list_value("['elem1', 'elem2', 'elem3']") == ["elem1", "elem2", "elem3"]
     assert processor._process_list_value("  ['elem1',   'elem2' , 'elem3']  ") == ["elem1", "elem2", "elem3"]
@@ -204,7 +207,6 @@ def test_process_list_value_function(processor):
     assert processor._process_list_value("[{'obj': 'val', 'obj2': 'val2'}, {}]") == [{'obj': 'val', 'obj2': 'val2'}, {}]
 
     # Test values that will not be successfully converted to list
-    assert processor._process_list_value("") == ""
     assert processor._process_list_value("[[]") == "[[]"
     assert processor._process_list_value("[elem1, elem2]") == "[elem1, elem2]"
     assert processor._process_list_value("elem1, elem2") == "elem1, elem2"
@@ -214,7 +216,10 @@ def test_process_list_value_function(processor):
 
 def test_process_dictionary_value_function(processor):
     # Test values that will be successfully converted to dictionary
+    assert processor._process_dictionary_value("") == {}
+    assert processor._process_dictionary_value(None) == {}
     assert processor._process_dictionary_value("{}") == {}
+    assert processor._process_dictionary_value("None") == {}
     assert processor._process_dictionary_value("{'key': 'value'}") == {"key": "value"}
 
     dict_as_str = "{'key1': 'value', 'key2': 'value'}"
@@ -248,7 +253,6 @@ def test_process_dictionary_value_function(processor):
     assert processor._process_dictionary_value(dict_as_str) == expected_value
 
     # Test values that will not be successfully converted to dictionary
-    assert processor._process_dictionary_value("") == ""
     assert processor._process_dictionary_value("{{}") == "{{}"
     assert processor._process_dictionary_value("{key1: value, key2: value}") == "{key1: value, key2: value}"
     assert processor._process_dictionary_value("  { key1: value, key2: value }  ") == "{ key1: value, key2: value }"
